"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("simpleutils");
const basicutil_1 = require("basicutil");
const child_1 = require("../child/child");
class AppObject extends child_1.Child {
    constructor(geneticCode) {
        super(Object.assign({ className: 'AppObject' }, geneticCode));
        this.checkPageFrame = false;
        this.checkView = false;
        this.checkNotification = false;
    }
    static getTypes() {
        return this.types;
    }
    static getInstance(geneticCode) {
        return new this(geneticCode);
    }
    static addType(type) {
        AppObject.addConstructor(type.getConstructor());
    }
    static addConstructor(constructor) {
        let className = constructor.name;
        if (AppObject.types === undefined) {
            AppObject.types = {};
        }
        if (AppObject.types[className] === undefined) {
            AppObject.types[className] = constructor;
            // console.log('className', className);
            // console.log('constructor', constructor);
        }
    }
    getConstructor() {
        return this.constructor;
    }
    getPageFrame() {
        if (!this.checkPageFrame) {
            this.setPageFrame();
        }
        return this.pageFrame;
    }
    getView() {
        if (!this.checkView) {
            this.setView();
        }
        return this.view;
    }
    getPage() {
        if (this.page === undefined) {
            this.setPage();
        }
        return this.page;
    }
    getPageBody() {
        return this.getView().getPageBody();
    }
    getHeader() {
        return this.getView().getHeader();
    }
    getFooter() {
        return this.getView().getFooter();
    }
    getNotification() {
        return this.getView().getNotification();
    }
    getNotificationName() {
        if (this.notificationName === undefined) {
            this.setNotificationName();
        }
        return this.notificationName;
    }
    clearProperty(property) { }
    generateElementStyleFromJSON(jSON, property) { }
    generateElementVarFromJSON(jSON, property) { }
    generateNotElementStyleFromJSON(jSON, property) {
        let _self = this;
        if (property === 'style') {
            // console.log('Prop is style');
            _self.generateFromJSON(jSON[property], jSONObjectType_1.JSONObjectType.Style);
        }
        else {
            // console.log('Prop is special');
            _self.generateElementVarFromJSON(jSON, property);
        }
    }
    generateArrayFromJSON(jSON, property) {
        let _self = this;
        _self.clearProperty(property);
        if (_self.getProperty(property).type !== undefined) {
            _self.generateElementsFromJSONArrayProperty(jSON, property);
        }
        else {
            _self.generateObjectsFromJSONArrayProperty(jSON, property);
        }
    }
    generateObjectsFromJSONArrayProperty(jSON, property) {
        let _self = this;
        jSON[property].forEach(element => {
            _self.generateObjectFromJSON(element, property);
        });
    }
    generateElementsFromJSONArrayProperty(jSON, property) {
        let _self = this;
        jSON[property].forEach(element => {
            _self.generateElementFromJSON(element, property);
        });
    }
    generateObjectFromJSON(jSON, property) {
        let _self = this;
        // console.log('START');
        // console.log('type', element.type);
        let geneticCode = { father: _self }; // , jSON: jSON}; fix: fazer uma maneira do populate ser chamado só depois do chonstrutor da child
        let object = AppObject.getTypes()[jSON.type];
        let properElement;
        if (object !== null && object !== undefined) {
            properElement = new object(geneticCode);
        }
        else {
            object = AppObject.getTypes()['ComponentGeneric'];
            properElement = new object(Object.assign({ specificName: jSON.type }, geneticCode));
        }
        // console.log('object', object);
        // console.log('properElement', properElement);
        properElement.populate(jSON); // remover depois do fix
        // _self[property].push(properElement);
    }
    generateElementFromJSON(jSON, property) {
        let _self = this;
        let properElement = new _self[property].type(_self);
        // console.log(properElement);
        properElement.populate(jSON);
        _self[property].push(properElement);
    }
    generateWithTypeFromJSON(jSON, property, type) {
        let _self = this;
        // console.log('Prop2');
        if (type === jSONObjectType_1.JSONObjectType.Style) {
            // console.log('Prop3 is var');
            _self.generateElementStyleFromJSON(jSON, property);
        }
        else {
            _self.generateNotElementStyleFromJSON(jSON, property);
        }
    }
    generateWithoutTypeFromJSON(jSON, property) {
        let _self = this;
        if (typeof jSON[property] === 'object') {
            _self.generatePropertyFromJSON(jSON, property);
        }
        else {
            // console.log('Prop is var:' + jSON[property]);
            _self.setProperty(property, jSON[property]);
        }
    }
    getProperty(property) {
        let _self = this;
        if (_self[property].constructor === Function &&
            _self[property]().constructor === Array) {
            return _self[property]();
        }
        return _self[property];
    }
    setProperty(property, value) {
        let _self = this;
        if (_self[property] && _self[property].constructor &&
            _self[property].constructor === Function &&
            _self[property]().constructor === Array) {
        }
        else
            _self[property] = value;
    }
    populateRegularPropertyFromJSON(jSON, property) {
        let _self = this;
        if (_self.getProperty(property).constructor === Array) {
            _self.generateArrayFromJSON(jSON, property);
        }
        else {
            _self[property].populate(jSON[property]);
            // _self[property].insert(_self);
        }
    }
    generateRegularPropertyFromJSON(jSON, property) {
        let _self = this;
        if (_self[property] === undefined) {
            _self[property] = jSON[property];
            // _self[property].insert(_self);
        }
        else {
            _self.populateRegularPropertyFromJSON(jSON, property);
        }
    }
    generatePropertyFromJSON(jSON, property) {
        let _self = this;
        // console.log('Prop is object');
        if (property === 'element') {
            // console.log('Prop is element');
            _self.generateFromJSON(jSON[property], jSONObjectType_1.JSONObjectType.Element);
            // // console.log('Prop is element OUT');
        }
        else {
            // console.log('Prop is regular');
            _self.generateRegularPropertyFromJSON(jSON, property);
        }
    }
    populatePropertyFromJSON(jSON, property, type) {
        let _self = this;
        // console.log('DEFINED!');
        if (!jSON.hasOwnProperty(property)) {
            return;
        }
        // console.log('TYPE:'+type);
        if (type) {
            _self.generateWithTypeFromJSON(jSON, property, type);
        }
        else {
            _self.generateWithoutTypeFromJSON(jSON, property);
        }
    }
    generateFromJSON(jSON, type) {
        let _self = this;
        for (let property in jSON) {
            // console.log('Prop:' + property);
            if (property !== undefined) {
                _self.populatePropertyFromJSON(jSON, property, type);
            }
        }
    }
    populate(jSON) {
        let _self = this;
        _self.renderBeforeUpdate();
        // console.log('GENERATE!');
        _self.generateFromJSON(jSON);
        _self.setProperties();
        _self.renderAfterUpdate();
    }
    setProperties() {
    }
    renderBeforeUpdate() {
    }
    renderAfterUpdate() {
    }
    setCurrentLanguage(language) {
        basicutil_1.Util.getInstance().setLanguage(language);
    }
    getCurrentLanguage() {
        return basicutil_1.Util.getInstance().getCurrentLanguage();
    }
    getArrayType(array) {
        return array.type;
    }
    getNameFromArrayName(arrayName) {
        return arrayName.split('array')[1];
    }
    setPageFrame() {
        this.checkPageFrame = true;
        this.pageFrame = this.seekFather('ComponentPageFrame');
    }
    setView() {
        this.checkView = true;
        this.view = this.seekFather('ComponentView');
    }
    setPage() {
        this.page = this.getPageBody().getNextName();
    }
    setNotificationName() {
        if (this.getNotification() !== undefined) {
            this.notificationName = this.getNotification().getNextName();
        }
    }
}
exports.AppObject = AppObject;
const jSONObjectType_1 = require("./jSONObjectType");
AppObject.addConstructor(AppObject);
//# sourceMappingURL=appObject.js.map